﻿B4R=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=3.31
@EndOfDesignText@
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'Public variables can be accessed from all modules.
	Public ESP As ESP8266
	Public ssid As String = "pmaker.biz"
	Public password As String = "password"
	
	Public MacArray(6) As Byte
	
	Public const rtcWr As Int = 1   'write to rtc memory
	Public const rtcRd As Int = 2   'read from rtc memory
	Public const rtcGetTime As Int = 3  'get RTC clock periods
	Public const rtcGetPeriod As Int = 4 'get RTC clock period
	Public varLocation As Int
	Public VarLong As Long
	Public rtcTimeLong As Long
	Public rtcPeriodLong As Long
	
	Public RSSI As Int
	Public ESPex As ESP8266extras
	Public ping As ESP8266Ping
	Public InternetConnected As Boolean
	Public Wifistatus As Byte
End Sub

Sub Get_MacAddress() As Byte()
	RunNative("getMac", Null)
	Return MacArray
End Sub
#if C
  #include <ESP8266WiFi.h>
  void getMac(B4R::Object* u) {
  WiFi.macAddress((Byte*)b4r_others::_macarray->data);
  }
#end if

'there are 128 x 4byte user memory locations for storage (at = 0 to 127)
'val holds the value to write or is ignored for the other 4 cases or is the deepsleep period in mS
'action is 0 - 3 for the 4 rtc functions available
Public Sub RTC(Action As Int, Val As ULong, At As ULong)
	varLocation = 64 + (At * 4)
	Select Action
		Case rtcWr
			If (varLocation < 64) Or (varLocation > 191) Then 
				Log("RTC.error.varLocation: ", varLocation)
				Return
			End If
			VarLong = Val
			RunNative("rtc_Write", Action)
		Case rtcRd
			If (varLocation < 64) Or (varLocation > 191) Then 
				Log("RTC.error.varLocation: ", varLocation)
				Return
			End If
			RunNative("rtc_Read", Action)
		Case rtcGetTime
			RunNative("rtc_GetTime", Action)
		Case rtcGetPeriod
			RunNative("rtc_GetPeriod", Action)
		Case Else
	End Select
End Sub

#if C
//#define RTCMEMORYSTART 64
//#define RTCMEMORYLEN 128
#include "user_interface.h" // this is for the RTC memory read/write functions
Long varLong;
ulong varlocation;
bool rtc_Write(B4R::Object* u){
   varLong = b4r_others::_varlong;
   varlocation = b4r_others::_varlocation;
   system_rtc_mem_write(varlocation, &varLong, sizeof(varLong));
   yield();
   }
bool rtc_Read(B4R::Object* u){
   varlocation = b4r_others::_varlocation;
   system_rtc_mem_read(varlocation, &varLong, sizeof(varLong));
   b4r_others::_varlong = varLong;
   yield();
   }
bool rtc_GetTime(B4R::Object* u){
   b4r_others::_rtctimelong = system_get_rtc_time();
   yield();
   }
bool rtc_GetPeriod(B4R::Object* u){ 
   b4r_others::_rtcperiodlong = system_rtc_clock_cali_proc();
   yield();
   }
#end if

'in percent
Public Sub Get_WiFI_RSSI As Int
	Check_Internet
	RunNative("GetRSSI", Null)
	'Log("WiFi, dB: " , RSSI)
	If InternetConnected = False Then
		Return -1
	Else If RSSI <= -100 Then
		Return 0
	Else If RSSI >= -50 Then
		Return 100
	Else
		Return 2 * (RSSI + 100)
	End If
End Sub

#if C
void GetRSSI (B4R::Object* o) {
b4r_others::_rssi = WiFi.RSSI();
}
#End if


Public Sub SetWifi_Power
	RunNative("SetWifiPower",Null)
End Sub

Public Sub ResetWiFi
	RunNative("StopAutoConnect",Null)
	RunNative("StopPersistent",Null)
End Sub
Public Sub SetupWiFi
	Dim name As String = ESPex.UniqueID
	RunNative("StartAutoConnect",Null)
	RunNative("StartPersistent",Null)
	RunNative("SetHostName", name)
End Sub

#if C
void SetWifiPower(B4R::Object* o) {
     WiFi.setOutputPower(20.5);  // set highest WiFi power and should be multiples of 0.25
}
void StopPersistent(B4R::Object* o) {
     WiFi.persistent(false);
}
void StartPersistent(B4R::Object* o) {
     WiFi.persistent(true);
}
void StopAutoConnect(B4R::Object* o) {
     WiFi.setAutoConnect(false);
}
void StartAutoConnect(B4R::Object* o) {
    WiFi.setAutoConnect(true);
}
  void SetHostName(B4R::Object* o) {
   B4R::Array* b = (B4R::Array*)B4R::Object::toPointer(o);
   char* c = (char*)b->data;
   WiFi.hostname(c);
  }
#end if

Public Sub restart (tag As Byte)
	Log("Restart")
	ESP.Restart
End Sub

Public Sub Go_Sleep(ms As ULong)
	Log("Go deep sleep for ", (ms / 1000), " secs")
	DeepSleep(ms)
End Sub

Private Sub DeepSleep(ms As ULong)
	RunNative("deepSleep", ms * 1000)
End Sub

Public Sub LightSleep_OFF
	Log("Light sleep is disabled")
	RunNative("DisableLightSleep", Null)
End Sub

Public Sub LightSleep_ON
	Log("Light sleep is enabled")
	RunNative("EnableLightSleep", Null)
End Sub

#if C
void deepSleep(B4R::Object* o) {
 //  WiFi.mode( WIFI_OFF );
 //  WiFi.forceSleepBegin();
 //  delay( 1 );
 //  ESP.deepSleep(o->toULong());
 //  system_deep_sleep_set_option(2);
   ESP.deepSleep(o->toULong(), WAKE_RF_DEFAULT);	//WAKE_RF_DISABLED );
}

void DisableLightSleep(B4R::Object* o) {
  // Disable light sleep
  wifi_set_sleep_type(NONE_SLEEP_T);
  delay(2000);
 }
void EnableLightSleep(B4R::Object* o) {
  // Enable light sleep
  wifi_set_sleep_type(LIGHT_SLEEP_T);
  delay(2000);
}
#end if

Public Sub Go_FromDeepSleep
	RunNative("FromDeepSleep", Null)
End Sub

#if C
void FromDeepSleep(B4R::Object* o) {
   WiFi.forceSleepWake();
   delay( 1 );

   // Bring up the WiFi connection
  //WiFi.mode( WIFI_STA );
}
#end if

Sub Save_Settings
	Main.eeprom.WriteBytes(GlobalStore.Slot1, 0)	'latest local IPб 4 bytes
	Main.eeprom.WriteBytes(GlobalStore.Slot2, 4)	'latest local IPб 4 bytes
	Main.eeprom.WriteBytes(GlobalStore.Slot3, 8)	'latest local IPб 4 bytes
	Main.eeprom.WriteBytes(GlobalStore.Slot4, 12)	'latest local IPб 4 bytes
	Log("Settings saved.")
End Sub

Sub Load_Settings
	Log("Loading settings...")
	Dim L(4) As Byte = Main.eeprom.ReadBytes(0, 4)
	GlobalStore.Put(1, L) 'latest local IP
	Log("Restored IP: ", ESPex.IP2str(L))
	
	Dim L(4) As Byte = Main.eeprom.ReadBytes(4, 4)
	GlobalStore.Put(2, L)
	Log("Restored Mask: ", ESPex.IP2str(L))
	
	Dim L(4) As Byte = Main.eeprom.ReadBytes(8, 4)
	GlobalStore.Put(3, L)
	Log("Restored Gateway: ", ESPex.IP2str(L))
	
	Dim L(4) As Byte = Main.eeprom.ReadBytes(12, 4)
	GlobalStore.Put(4, L)
	Log("Restored DNS: ", ESPex.IP2str(L))
End Sub

Public Sub Check_Internet
	InternetConnected = ping.PingHost("rtz63.online", 2)
	Log("InternetConnected = ", InternetConnected)
End Sub

Public Sub Get_WiFiStatus
	'WL_NO_SHIELD	255
	'WL_IDLE_STATUS	0
	'WL_NO_SSID_AVAIL	1
	'WL_SCAN_COMPLETED	2
	'WL_CONNECTED	3
	'WL_CONNECT_FAILED	4
	'WL_CONNECTION_LOST	5
	'WL_DISCONNECTED	6
	RunNative("GetWiFiStatus", Null)
End Sub

#if C
void GetWiFiStatus (B4R::Object* o) {
b4r_others::_wifistatus = WiFi.status();
}
#End if